"""Angets (Ankha's Gets): Functions for user input."""
import importlib.metadata

__version__: str = importlib.metadata.version('angets')

from angets._core import (
    get_non_empty_str, get_constrained_number, get_float, get_constrained_float, get_positive_float,
    get_non_negative_float, get_int, get_constrained_int, get_positive_int, get_non_negative_int, get_confirmation,
    get_date
)
import angets._decorators as decorators
import angets._helpers as helpers
import angets._exceptions as exceptions
